#include "MainWindow.h"
#include "Game.h"
#include "maps.h"

Game::Game( MainWindow& wnd )
	:
	wnd( wnd ),
	gfx( wnd )
{

    //init array for randomized floor tileset (3 variants), drawn in Game::draw_floor();
    for (int i = 0; i < map_height; i++) {
        for (int j = 0; j < map_width; j++) {
            int r = rand() % 3;
            rand_floor[i][j] = r;
        }
    }

    //initializing level specifics (level 1 in the constructor function only)
    level_init(current_level);
}

void Game::Go()
{
	gfx.BeginFrame();	
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}



void Game::UpdateModel()
{
    //loop when game is running
    if (isStarted && !gameover)
    {
        player.move(wnd);
        player.keep_boundaries(boundary, gfx);

        for (int i = 0; i < NUM_WATER; i++) {
            if (water[i].isInitialized()) {
                if (player.collides(water[i].getX(), water[i].getY(), WATER_WIDTH, WATER_HEIGHT))
                {
                    player.setSpeed(1);
                }
                else
                {
                    player.setSpeed(player.default_speed);
                }
            }
        }

        for (int i = 0; i < NUM_MOVING_FIRES; i++) {
            fire[i].move();
        }
    }  

    //finishing the level by colliding with the "level exit" object
    if (player.collides_with_level_exit(level_exit.getX(), level_exit.getY()) && level_exit.isInitialized())
    {
        current_level_completed = true;
    }

    if (!isStarted)
    // "press any key to start the game" 
    { 
        if (wnd.kbd.ReadChar() || wnd.kbd.KeyIsPressed(VK_UP) || wnd.kbd.KeyIsPressed(VK_DOWN) || wnd.kbd.KeyIsPressed(VK_LEFT) || wnd.kbd.KeyIsPressed(VK_RIGHT))
        {
            // = game is running now
            isStarted = true; 
        }
    }

    if (gameover)
    {
        wnd.kbd.Flush();

        if (wnd.kbd.KeyIsPressed(VK_RETURN))
        {
            gameover = false;
            isStarted = false;
            level_init(current_level);
        }
    }

    if (current_level_completed)
    {
        wnd.kbd.Flush();
        if (wnd.kbd.KeyIsPressed(VK_RETURN))
        {
            if ((current_level + 1) <= LEVELS_TOTAL)
            {              
                current_level++;
                reset_map();
                level_init(current_level);
                current_level_completed = false;
                isStarted = false; // showing the intro screen for the level
            }
            else
            {   // victory screen, completed all levels
                wnd.Kill();
            }
        }
    }

    // ESC terminates the program
    if (wnd.kbd.KeyIsPressed(VK_ESCAPE))
    {
        wnd.Kill();
    }
}

void Game::ComposeFrame()
{
    //drawing floor tiles with pseudo-random pattern
    draw_floor();

    //drawing all the bricks and checking collision with player
    for (int i = 0; i < NUM_BRICKS; i++) {
        if (bricks[i].isInitialized()) {
            bricks[i].draw(gfx);
            player.check_collision_bricks(bricks[i].getX(), bricks[i].getY());
        }
    }

    //drawing the tileset
    for (int i = 0; i < NUM_TILES; i++) {
        if (tiles[i].isInitialized()) {
            tiles[i].draw(gfx);
        }
    }

    //drawing the water
    for (int i = 0; i < NUM_WATER; i++) {
        if (water[i].isInitialized()) {
            water[i].draw(gfx);
        }
    }

    //drawing moving fires and checking collision with player
    for (int i = 0; i < NUM_MOVING_FIRES; i++) {
        if (fire[i].isInitialized()) {
            fire[i].draw(gfx);

    //ending the game if player touches moving fires
            if (player.collides_with_moving_fires(fire[i].getX(),fire[i].getY())) { 
                gameover = true;
            }
        }
    }

    //drawing a level exit
    if (level_exit.isInitialized()) { level_exit.draw(gfx); }

    //drawing player
    player.draw(gfx);
    
    //intro screen at start. Pressing any key results in game start (@ UpdateModel() )
    if (!isStarted)
    {
        draw_intro_screen();
    }

    if (gameover)
    {
        draw_gameover_screen();
    }

    if (current_level_completed)
    {
        draw_level_completed();
    }
}

void Game::draw_intro_screen()
{
    gfx.DrawSprite(0, 0, surface_intro_screen, SpriteEffect::Copy{});
}

void Game::draw_gameover_screen()
{
    gfx.DrawSprite(0, 0, surface_game_over, SpriteEffect::Copy{});
}

void Game::draw_level_completed()
{
    gfx.DrawSprite(0, 0, surface_level_completed, SpriteEffect::Copy{});
}

void Game::draw_floor()
{
    for (int i = 0; i < map_height; i++) {
        for (int j = 0; j < map_width; j++) {
            switch (rand_floor[i][j]) {
            case 0:
                gfx.DrawSprite(j * TILE_WIDTH, i * TILE_HEIGHT, surface_floor_1, SpriteEffect::Copy{});
                break;
            case 1:
                gfx.DrawSprite(j * TILE_WIDTH, i * TILE_HEIGHT, surface_floor_2, SpriteEffect::Copy{});
                break;
            case 2:
                gfx.DrawSprite(j * TILE_WIDTH, i* TILE_HEIGHT, surface_floor_3, SpriteEffect::Copy{});
                break;
            default:
                gfx.DrawSprite(j * TILE_WIDTH, i * TILE_HEIGHT, surface_floor_1, SpriteEffect::Copy{});
                break;
            }
        }
    }
}